/**
 * JASMINe
 * Copyright (C) 2009-2010 Bull S.A.S.
 * Contact: jasmine@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: DataManagementModelLocator.as 6019 2010-01-27 08:25:37Z fornacif $
 * --------------------------------------------------------------------------
 */
package org.ow2.jasmine.monitoring.eos.datamanagement.model
{
    import com.adobe.cairngorm.model.ModelLocator;
    
    import mx.collections.ArrayCollection;
    import mx.controls.*;
    import mx.messaging.events.MessageEvent;
    import mx.messaging.messages.AsyncMessage;
    import mx.utils.UIDUtil;
    
    import org.granite.gravity.Consumer;
    import org.ow2.jasmine.monitoring.eos.datamanagement.business.*;
    import org.ow2.jasmine.monitoring.eos.datamanagement.vo.PurgeTaskVO;
    
    /**
     * The model locator for the data management module.
     */
    [Bindable]
    public class DataManagementModelLocator implements ModelLocator
    {
        /**
         * The unique ID of this component
         *
         * @internal
         *   Used to prevent a Cairngorm issue: when a command event is dispatched,
         * every controller that registered this event type receives it, even if
         * located in another module. To prevent this from happening and triggering
         * multiple severe unexpected concurrence bugs, each event dispatched is
         * postfixed with this unique ID.
         */
        public var componentID:String = UIDUtil.createUID();
        
        /**
         * Unique instance of this locator.
         */
        private static var model:DataManagementModelLocator = null;
        
        
        //
        //    Delegates
        //
        private var configurePurgeTaskDelegate:IConfigurePurgeTaskDelegate = null; 
        private var listPurgeTasksDelegate:IListPurgeTasksDelegate = null;
        private var removePurgeTaskDelegate:IRemovePurgeTaskDelegate = null;
        private var startPurgeTaskDelegate:IStartPurgeTaskDelegate = null;
        private var pausePurgeTaskDelegate:IPausePurgeTaskDelegate = null;
        
        /**
         * List of created purge tasks.
         */
        private var _purgeTasks:ArrayCollection;
        
        /**
         * Consumer of the jasminePurgeTask topic.
         */
        private var consumer:Consumer = null;
        
        // functions
        public function DataManagementModelLocator()
        {
            super();
            
            _purgeTasks = new ArrayCollection();
            
            if (model != null) {
                throw new Error("Only one DataManagementModelLocator has to be set");
            }
        }
        
        public static function getInstance() : DataManagementModelLocator {
            if (DataManagementModelLocator.model == null) {
                DataManagementModelLocator.model = new DataManagementModelLocator();
            }
            
            return DataManagementModelLocator.model;
        }
        
        // setters
        
        
        // getters delegates
        
        public function getConfigurePurgeTaskDelegate():IConfigurePurgeTaskDelegate {
            if (this.configurePurgeTaskDelegate == null) {
                this.configurePurgeTaskDelegate = new ConfigurePurgeTaskDelegate();
            }
            return this.configurePurgeTaskDelegate;
        }
        
        public function getListPurgeTasksDelegate():IListPurgeTasksDelegate {
            if (this.listPurgeTasksDelegate == null) {
                this.listPurgeTasksDelegate = new ListPurgeTasksDelegate();
            }
            return this.listPurgeTasksDelegate;
        }
        
        public function getRemovePurgeTaskDelegate():IRemovePurgeTaskDelegate {
            if (this.removePurgeTaskDelegate == null) {
                this.removePurgeTaskDelegate = new RemovePurgeTaskDelegate();
            }
            return this.removePurgeTaskDelegate;
        }
        
        public function getStartPurgeTaskDelegate():IStartPurgeTaskDelegate {
            if (this.startPurgeTaskDelegate == null) {
                this.startPurgeTaskDelegate = new StartPurgeTaskDelegate();
            }
            return this.startPurgeTaskDelegate;
        }
        
        public function getPausePurgeTaskDelegate():IPausePurgeTaskDelegate {
            if (this.pausePurgeTaskDelegate == null) {
                this.pausePurgeTaskDelegate = new PausePurgeTaskDelegate();
            }
            return this.pausePurgeTaskDelegate;
        }
        
        /**
        * Subscribe to the jasminePurgeTask topic to receive purge task modifications:
        * Number of removed events, last purge task execution, ... 
        * */
        public function subscribe():void {
            consumer = new Consumer();          
            consumer.destination = "gravity";
            consumer.topic = "jasminePurgeTask";
            consumer.subscribe();
            consumer.addEventListener(MessageEvent.MESSAGE, purgeTaskChanged);
        }
        
        public function disconnect():void {
            if (consumer != null) {
                consumer.disconnect();
                consumer = null;
            }
            
            _purgeTasks.removeAll();
        }
        
        private function purgeTaskChanged(event:MessageEvent):void {
            var msg:AsyncMessage = event.message as AsyncMessage; 
            if (msg.body is PurgeTaskVO) {
                var newPurgeTask:PurgeTaskVO = msg.body as PurgeTaskVO;
    
                for each(var purgeTask:PurgeTaskVO in purgeTasks) {                   
                    if (newPurgeTask.id == purgeTask.id) {
                        // Update possible modified properties
                        purgeTask.started = newPurgeTask.started;
                        purgeTask.lastExecutionDate = newPurgeTask.lastExecutionDate;
                        purgeTask.removedEvents = newPurgeTask.removedEvents;
                        break;
                    }
                }
            }
        }
        
        public function addPurgeTask(purgeTask:PurgeTaskVO):void {
            this._purgeTasks.addItem(purgeTask);
        }
        
        public function removePurgeTask(purgeTaskId:Number):void {
            var indexOfPurgeTaskToRemove:int;
            for (var i:Number = 0; i < purgeTasks.length; i++) {
                var purgeTask:PurgeTaskVO = purgeTasks[i];
                if (purgeTaskId == purgeTask.id) {
                    indexOfPurgeTaskToRemove = i;
                    break;
                }
            }
            this._purgeTasks.removeItemAt(indexOfPurgeTaskToRemove);
        }
        
        public function get purgeTasks():ArrayCollection {
            return this._purgeTasks;
        }   
        
    }
}